-- ProfessionTabs v1.2 by Beoko

local IsCurrentSpell = IsCurrentSpell
local format = string.format
local next = next
local ranks = PROFESSION_RANKS
local _, class = UnitClass("player")
local tabs, spells = {}, {}

local handler = CreateFrame("Frame")
handler:SetScript("OnEvent", function(self, event) self[event](self, event) end)
handler:RegisterEvent("TRADE_SKILL_SHOW")
handler:RegisterEvent("TRADE_SKILL_CLOSE")
handler:RegisterEvent("TRADE_SHOW")
handler:RegisterEvent("SKILL_LINES_CHANGED")
handler:RegisterEvent("CURRENT_SPELL_CAST_CHANGED")

local defaults = {
	-- Primary Professions
	[171] = { true, false }, -- 炼金
	[164] = { true, false }, -- 锻造
	[333] = { true, true }, -- 附魔
	[202] = { true, false }, -- 工程
	[182] = { false, false }, -- 采药
	[773] = { true, true }, -- 铭文
	[755] = { true, true }, -- 珠宝
	[165] = { true, false }, -- 制皮
	[186] = { true, false }, -- 采矿
	[393] = { false, false }, -- 剥皮
	[197] = { true, false }, -- 裁缝
	
	-- Secondary Professions
	[794] = { false, false }, -- 考古
	[185] = { true, true }, -- 烹饪
	[129] = { true, false }, -- 急救
	[356] = { false, false }, -- 钓鱼
}

if class == "DEATHKNIGHT" then spells[ #spells + 1 ] = 53428 end -- Runeforging
if class == "ROGUE" then spells[ #spells + 1 ] = 1804 end -- Pick Lock

local function UpdateSelectedTabs(object)
	if not handler:IsEventRegistered("CURRENT_SPELL_CAST_CHANGED") then
		handler:RegisterEvent("CURRENT_SPELL_CAST_CHANGED")
	end
	
	for index = 1, #tabs[object] do
		local tab = tabs[object][index]
		tab:SetChecked(IsCurrentSpell(tab.name))
	end
end

local function ResetTabs(object)
	for index = 1, #tabs[object] do
		tabs[object][index]:Hide()
	end
	
	tabs[object].index = 0
end

local function UpdateTab(object, name, rank, texture)
	local index = tabs[object].index + 1
	local tab = tabs[object][index] or CreateFrame("CheckButton", nil, object, "SpellBookSkillLineTabTemplate SecureActionButtonTemplate") 
	
	tab:ClearAllPoints()
	tab:SetPoint("TOPLEFT", object, "TOPRIGHT", object == ATSWFrame and -32 or 0, 16 + -44 * index)
	tab:SetNormalTexture(texture)
	tab:SetAttribute("type", "spell")
	tab:SetAttribute("spell", name)
	tab:Show()
	
	tab.name = name
	tab.tooltip = rank and rank ~= "" and format("%s (%s)", name, rank) or name
	
	tabs[object][index] = tabs[object][index] or tab
	tabs[object].index = tabs[object].index + 1
end

local function GetProfessionRank(currentSkill)
	for index = #ranks, 1, -1 do
		local requiredSkill, title = ranks[index][1], ranks[index][2]
		
		if currentSkill >= requiredSkill then
			return title
		end
	end
end

local function HandleProfession(object, professionID)
	if professionID then
		local _, _, currentSkill, _, numAbilities, offset, skillID = GetProfessionInfo(professionID)
		
		if defaults[skillID] then
			for index = 1, numAbilities do
				if defaults[skillID][index] then
					local name = GetSpellBookItemName(offset + index, "profession")
					local rank = GetProfessionRank(currentSkill)
					local texture = GetSpellBookItemTexture(offset + index, "profession")
					
					if name and texture then	--rank and 8.0去掉
						UpdateTab(object, name, rank, texture)
					end
				end
			end
		end
	end
end

local function HandleTabs(object)
	tabs[object] = tabs[object] or {}
	
	if InCombatLockdown() then
		handler:RegisterEvent("PLAYER_REGEN_ENABLED")
	else
		local firstProfession, secondProfession, archaeology, fishing, cooking, firstAid = GetProfessions()
		
		ResetTabs(object)
		
		HandleProfession(object, firstProfession)
		HandleProfession(object, secondProfession)
		HandleProfession(object, archaeology)
		HandleProfession(object, fishing)
		HandleProfession(object, cooking)
		HandleProfession(object, firstAid)
		
		for index = 1, #spells do
			if IsSpellKnown( spells[index] ) then
				local name, rank, texture = GetSpellInfo( spells[index] )
				UpdateTab(object, name, rank, texture)
			end
		end
	end
	
	UpdateSelectedTabs(object)
end

function handler:TRADE_SKILL_SHOW(event)
	local owner = ATSWFrame or MRTSkillFrame or SkilletFrame or TradeSkillFrame
	
	if IsAddOnLoaded("TradeSkillDW") and owner == TradeSkillFrame then
		self:UnregisterEvent(event)
	else
		HandleTabs(owner)
		self[event] = function() for object in next, tabs do UpdateSelectedTabs(object) end end
	end
end

function handler:TRADE_SKILL_CLOSE(event)
	for object in next, tabs do 
		if object:IsShown() then
			UpdateSelectedTabs(object)
		end
	end
end

function handler:TRADE_SHOW(event)
	local owner = TradeFrame
	
	HandleTabs(owner)
	self[event] = function() UpdateSelectedTabs(owner) end
end

function handler:PLAYER_REGEN_ENABLED(event)
	self:UnregisterEvent(event)
	
	for object in next, tabs do HandleTabs(object) end
end

function handler:SKILL_LINES_CHANGED()
	for object in next, tabs do HandleTabs(object) end
end

function handler:CURRENT_SPELL_CAST_CHANGED(event)
	local numShown = 0
	
	for object in next, tabs do 
		if object:IsShown() then
			numShown = numShown + 1
			UpdateSelectedTabs(object)
		end
	end
	
	if numShown == 0 then self:UnregisterEvent(event) end
end